
#include <stdlib.h>
#include <stdio.h>
#include <aalib.h>

#include <SDL/SDL.h>

#include "fb.h"
#include "input.h"
#include "rc.h"
#include "ter8.h"
struct fb fb;

static int use_yuv = -1;
static int fullscreen = 1;
static int use_altenter = 1;
static char Xstatus, Ystatus;

static int vmode[3] = { 0, 0, 16 };


rcvar_t vid_exports[] =
{
	RCV_VECTOR("vmode", &vmode, 3),
	RCV_BOOL("yuv", &use_yuv),
	RCV_BOOL("fullscreen", &fullscreen),
	RCV_BOOL("altenter", &use_altenter),
	RCV_END
};

/* keymap - mappings of the form { scancode, localcode } - from sdl/keymap.c */
extern int keymap[][2];

aa_context *context;
aa_palette palette;
aa_renderparams *params;
int value_last;

byte * buffer;
int x_offset;

void vid_init()
{

    if (!vmode[0] || !vmode[1]){
        int scale = rc_getint("scale");
		if (scale < 1) scale = 1;
		vmode[0] = 160 * scale;
		vmode[1] = 144 * scale;
    }
	
    joy_init();
     
    vmode[0] = 160;
    vmode[1] = 144;
     
     
    aa_defparams.font           = &aa_fontgl; 
    aa_defparams.minwidth       = vmode[0];
    aa_defparams.minheight      = vmode[1];
    aa_defparams.width          = vmode[0];
    aa_defparams.height         = vmode[1];
     
    /*
     * aa_recommendhidisplay("slang");
    */
     context = aa_autoinit(&aa_defparams);
    
    if(context == NULL) {
        fprintf(stderr,"Cannot initialize AA-lib. Sorry\n");
        exit(1);
    }
     
    aa_hidecursor(context);
    params = aa_getrenderparams(); 
    aa_autoinitkbd(context,AA_SENDRELEASE);

	
    if (SDL_Init(SDL_INIT_VIDEO))
		die("SDL: Couldn't initialize SDL: %s\n", SDL_GetError());


    
    printf("Set aa window %ix%i\n",aa_scrwidth(context), aa_scrheight(context)); 
    

    fb.w        = vmode[0] * 2;
    fb.h        = vmode[1];
    fb.pitch    = aa_scrwidth(context)  * 2;
    fb.pelsize  = 2;
    fb.ptr      = context->imagebuffer;
    fb.indexed  = 1;
    fb.cc[0].r  = fb.cc[1].r = fb.cc[2].r = 8;
    fb.cc[0].l  = fb.cc[1].l = fb.cc[2].l = 0;
    fb.enabled = 1;
    fb.dirty = 1;
	
    if (  aa_scrwidth(context) > vmode[0])
        x_offset = (aa_scrwidth(context) - vmode[0]);
    else
        x_offset = 0;
     
    x_offset +=  aa_scrwidth(context); 
}


void vid_setpal(int i, int r, int g, int b)
{
    aa_setpalette(palette, i, r, g, b);
    
}

void vid_preinit()
{
}

void vid_close()
{
    aa_close(context);
    fb.enabled = 0;
}

void vid_settitle(char *title)
{

}

void vid_begin()
{     
    fb.ptr = context->imagebuffer + x_offset;

}

void vid_end()
{
    aa_renderpalette(context, palette, params, 0, 0, aa_scrwidth(context), aa_scrheight(context)); 
    aa_flush(context);  
}


void ev_poll()
{
    
    event_t ev;
    int     value;
    value = aa_getevent(context,0);
    
    if ( value == AA_NONE )
        return;
    
    if ( (char)value == 'q' )
            exit(1);
    
    /*
    printf("Poll got: '%i' \n",value);
    */
    if ( value <= AA_RELEASE ) {
        ev.type = EV_PRESS;
        /*
         * printf("PRESS\n");
         */
    } else {
        value &= ~AA_RELEASE;
        ev.type = EV_RELEASE;
        /*
         * printf("RELEASE\n");
         */
    }
    
    if ( value == AA_UP){
        value = K_UP;
        /*
         * printf("UP\n");
         */
    } else if ( value == AA_DOWN) {
        value = K_DOWN;
        /*
         * printf("DOWN\n");
         */
    } else if ( value == AA_RIGHT) {
        /* 
         * printf("RIGHT\n");
         */
        value = K_RIGHT;
    } else if ( value == AA_LEFT) {
        /*
         * printf("LEFT\n");
         */
        value = K_LEFT;
    }
        
    ev.code = value;
    ev_postevent(&ev);
}


